package de.mdelab.mlsdm.interpreter.debug.ui;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.ISourceLocator;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.interpreter.debug.MLSDMElementUuidCalculator;
import de.mdelab.mlsdm.interpreter.debug.client.MLSDMDebugClient;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.MLSDMDebugUiConstants;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view.MLSDMBreakpointsManager;
import de.mdelab.mlsdm.interpreter.debug.ui.instanceGraph.MLSDMInstanceGraphEditor;
import de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor.MLSDMEditorManager;
import de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor.StoryDiagramResourceFilter;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugTarget;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugThread;
import de.mdelab.sdm.interpreter.code.debug.ui.instanceGraph.SDInstanceGraphEditor;
import de.mdelab.sdm.interpreter.code.debug.ui.instanceGraph.SDInstanceGraphResourceSetEditorInput;
import de.mdelab.sdm.interpreter.code.debug.ui.storyDiagramEditor.SDEditorManager;
import de.mdelab.sdm.interpreter.core.debug.client.SDDebugClient;

public class MLSDMDebugTarget extends SDDebugTarget<MLElementWithUUID, Activity>
{

	public MLSDMDebugTarget(final ILaunch launch, final String debugServerAdress, final int debugServerPort) throws CoreException
	{
		super(launch, debugServerAdress, debugServerPort);
		MLSDMBreakpointsManager.getModelProvider().setDebugTarget(this);
	}

	@Override
	protected ISourceLocator createSDDebugSourceLocator()
	{
		return new MLSDMDebugSourceLocator(this);
	}

	@Override
	protected SDDebugClient<MLElementWithUUID, Activity> createSDDebugClient()
	{
		final MLSDMDebugClient debugClient = new MLSDMDebugClient();

		return debugClient;
	}

	@Override
	protected void configureDebugClient(final SDDebugClient<MLElementWithUUID, Activity> debugClient)
	{
		/*
		 * Add a filter so that storydiag files are not synchronized
		 */
		debugClient.getSynchronizerAdapter().addResourceFilter(new StoryDiagramResourceFilter());
		debugClient.getSynchronizerAdapter().getEObjectUUIDManager().setEObjectUuidCalculator(new MLSDMElementUuidCalculator());
	}

	@Override
	protected SDInstanceGraphEditor createSDInstanceGraphEditor()
	{
		return new MLSDMInstanceGraphEditor();
	}

	@Override
	protected SDEditorManager<MLElementWithUUID> createSDEditorManager()
	{
		return new MLSDMEditorManager(this);
	}

	@Override
	protected SDDebugThread<MLElementWithUUID, Activity> createSDDebugThread(final SDDebugClient<MLElementWithUUID, Activity> debugClient)
	{
		return new MLSDMDebugThread(this, debugClient);
	}

	@Override
	protected void openInstanceGraphEditor()
	{
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				try
				{
					/*
					 * Close existing editors
					 */
					final IEditorReference[] editors = page.findEditors(null, MLSDMDebugUiConstants.INSTANCE_TREE_GRAPH_EDITOR_ID,
							IWorkbenchPage.MATCH_ID);
					page.closeEditors(editors, false);

					/*
					 * Create resource set editor input
					 */
					final SDInstanceGraphResourceSetEditorInput input = new SDInstanceGraphResourceSetEditorInput(MLSDMDebugTarget.this
							.getDebugClient().getDebuggerResourceSet());

					/*
					 * Attach adapter to listen for notifications
					 */
					MLSDMDebugTarget.this.getDebugClient().getDebuggerResourceSet().eAdapters().add(new EContentAdapter()
					{
						@Override
						public void notifyChanged(final org.eclipse.emf.common.notify.Notification notification)
						{
							super.notifyChanged(notification);

							if (!notification.isTouch())
							{
								MLSDMDebugTarget.this.fireChangeEvent(DebugEvent.CONTENT);
							}
						};
					});

					MLSDMDebugTarget.this.setInstanceGraphEditor((MLSDMInstanceGraphEditor) page.openEditor(input,
							MLSDMDebugUiConstants.INSTANCE_TREE_GRAPH_EDITOR_ID));

					// SDDebugTarget.this.logger.debug("opened instance graph editor, editing domain: "
					// +
					// SDDebugTarget.this.instanceGraphEditor.getEditingDomain());
				}
				catch (final PartInitException e)
				{
					throw new RuntimeException("Could not open instance graph editor.", e);
				}
			}
		});
	}

	@Override
	protected void closeInstanceGraphEditor()
	{
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				final IEditorReference[] editors = page.findEditors(null, MLSDMDebugUiConstants.INSTANCE_TREE_GRAPH_EDITOR_ID,
						IWorkbenchPage.MATCH_ID);

				page.closeEditors(editors, true);
			}
		});
	}

	@Override
	public void terminate() throws DebugException
	{
		super.terminate();
		MLSDMBreakpointsManager.getModelProvider().resetDebugTarget();
	}
}
