package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints;

import java.net.URL;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.AbstractDecorator;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget;
import org.eclipse.gmf.runtime.draw2d.ui.mapmode.MapModeUtil;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.osgi.framework.Bundle;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlexpressions.MLStringExpression;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityFinalNode;
import de.mdelab.mlsdm.DecisionMergeNode;
import de.mdelab.mlsdm.FlowFinalNode;
import de.mdelab.mlsdm.InitialNode;
import de.mdelab.mlsdm.StoryNode;
import de.mdelab.mlsdm.diagram.custom.edit.parts.CustomActivityEdgeGuardConstraintLabelEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryNodeStoryNodeElementsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternStoryPatternConstraintsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.WrappingLabel4EditPart;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view.BreakpointsManagerEvent;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view.IBreakpointsManagerListener;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view.MLSDMBreakpointsManager;
import de.mdelab.mlstorypatterns.StoryPatternLink;
import de.mdelab.mlstorypatterns.StoryPatternObject;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugUiConstants;
import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;
//import de.mdelab.mlsdm.diagram.edit.parts.ActivityEdgeGuardConstraintLabelEditPart;
//import de.mdelab.mlsdm.diagram.edit.parts.StoryActionNodeStoryActionNodeConstraintsCompartmentEditPart;

public class BreakpointDecorator extends AbstractDecorator implements IBreakpointsManagerListener
{

	private boolean	subscribed	= false;

	private String	uuid;

	public BreakpointDecorator(final IDecoratorTarget decoratorTarget)
	{
		super(decoratorTarget);
	}

	@Override
	public void activate()
	{
		this.refresh();
	}

	@Override
	public void refresh()
	{
		this.removeDecoration();

		final EditPart editPart = (EditPart) this.getDecoratorTarget().getAdapter(EditPart.class);

		final View view = (View) this.getDecoratorTarget().getAdapter(View.class);

		if (!(view.getElement() instanceof MLElementWithUUID) || (view == null))
		{
			return;
		}

		final MLElementWithUUID element = (MLElementWithUUID) view.getElement();
		String uuid = "";

		if (editPart instanceof WrappingLabel4EditPart/* ActivityEdgeGuardConstraintLabelEditPart */)
		{
			if (((ActivityEdge) element).getGuardExpression() == null)
			{
				return;
			}
			uuid = ((ActivityEdge) element).getGuardExpression().getUuid();
		}
		else if (editPart instanceof StoryPatternStoryPatternConstraintsCompartmentEditPart)
		{
			if (((StoryNode) element).getStoryPattern().getConstraints().isEmpty())
			{
				return;
			}
			uuid = ((StoryNode) element).getStoryPattern().getConstraints().get(0).getUuid();
		}
		else if (editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart)
		{
			if (((StoryPatternObject) element).getConstraints().isEmpty())
			{
				return;
			}
			uuid = ((StoryPatternObject) element).getConstraints().get(0).getUuid();
		}
		else if (editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart)
		{
			if (((StoryPatternObject) element).getAttributeAssignments().isEmpty())
			{
				return;
			}
			uuid = ((StoryPatternObject) element).getAttributeAssignments().get(0).getUuid();
		}
		else
		{
			uuid = element.getUuid();
		}

		final IBreakpoint[] breakpoints = DebugPlugin.getDefault().getBreakpointManager()
				.getBreakpoints(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER);

		// look if breakpoint is intended for my decorator target
		IBreakpoint breakpoint = null;

		for (final IBreakpoint breakpoint2 : breakpoints)
		{
			if (((SDDebugBreakpoint) breakpoint2).getElementUUID().equals(uuid))
			{
				breakpoint = breakpoint2;
				this.uuid = uuid;
				break;
			}
		}

		if (breakpoint != null)
		{
			URL url = null;
			final String PLUGIN_ID = "de.mdelab.mlsdm.interpreter.debug.ui";
			final Bundle bundle = Platform.getBundle(/* SDDebugUiPlugin. */PLUGIN_ID);

			try
			{
				if (!this.subscribed)
				{
					MLSDMBreakpointsManager.getModelProvider().addListener(this);
					this.subscribed = true;
				}

				MLSDMBreakpointsManager.getModelProvider().checkBreakpointLocation(view.eResource().getURI().toString(),
						(SDDebugBreakpoint) breakpoint);

				if (breakpoint.isEnabled())
				{
					url = bundle.getEntry("icons/enabled/breakpoint.gif");
				}
				else
				{
					url = bundle.getEntry("icons/disabled/breakpoint.gif");
				}
			}
			catch (final CoreException e)
			{
				e.printStackTrace();
			}

			final ImageDescriptor imageDesc = ImageDescriptor.createFromURL(url);
			final Image image = imageDesc.createImage();
			if ((editPart instanceof CustomActivityEdgeGuardConstraintLabelEditPart)
					|| (editPart instanceof StoryNodeStoryNodeElementsCompartmentEditPart)
					|| (editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart)
					|| (editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart))
			{
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.WEST,
						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(0), false));
			}
			else if ((element instanceof ActivityEdge) || (element instanceof StoryPatternLink))
			{
				this.setDecoration(this.getDecoratorTarget().addConnectionDecoration(image, 50, false));

			}
			else if ((element instanceof InitialNode) || (element instanceof FlowFinalNode) || (element instanceof ActivityFinalNode))
			{
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.SOUTH_WEST,
						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-8), false));
			}
			else if ((element instanceof DecisionMergeNode) || (element instanceof InitialNode) || (element instanceof FlowFinalNode)
					|| (element instanceof ActivityFinalNode))
			{
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.CENTER,
						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-4), false));

			}
			else if (element instanceof StoryPatternObject)
			{
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.NORTH_EAST, -1, false));
			}
			else if (element instanceof MLStringExpression)
			{
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.EAST,
						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(0), false));
				// setDecoration(getDecoratorTarget().addConnectionDecoration(image,
				// 50, false));
			}
			else
			{
				// ((StoryActionNodeNameEditPart)editPart.getChildren().get(0)).getEditPolicy(key)
				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.NORTH_EAST,
						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-4), false));
				// this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image,
				// IDecoratorTarget.Direction.WEST,
				// MapModeUtil.getMapMode(((AbstractGraphicalEditPart)editPart).getFigure()).DPtoLP(-4),
				// false));

			}
		}
		else
		{
			if (this.subscribed)
			{
				MLSDMBreakpointsManager.getModelProvider().removeListener(this);
				// System.out.println("decorator unsubscribed");
				this.subscribed = false;
				uuid = null;
			}
		}
	}

	@Override
	public void breakpointChanged(final BreakpointsManagerEvent event)
	{

		if (event.getItem().getUuid().equals(this.uuid))
		{
			final View view = (View) this.getDecoratorTarget().getAdapter(View.class);
			try
			{
				final String string = view.eResource().getURI().toString();
				if (string != null)
				{
					this.refresh();
				}
			}
			catch (final Exception e)
			{
				MLSDMBreakpointsManager.getModelProvider().removeListener(this);
			}
		}
	}
}
