package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gmf.runtime.common.core.service.AbstractProvider;
import org.eclipse.gmf.runtime.common.core.service.IOperation;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IPrimaryEditPart;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.CreateDecoratorsOperation;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorProvider;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget;
import org.eclipse.gmf.runtime.notation.View;

import de.mdelab.mlexpressions.MLStringExpression;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityFinalNode;
import de.mdelab.mlsdm.DecisionMergeNode;
import de.mdelab.mlsdm.ExpressionActivityNode;
import de.mdelab.mlsdm.FlowFinalNode;
import de.mdelab.mlsdm.InitialNode;
import de.mdelab.mlsdm.StoryNode;
import de.mdelab.mlsdm.diagram.custom.edit.parts.CustomActivityEdgeGuardConstraintLabelEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternStoryPatternConstraintsCompartmentEditPart;
import de.mdelab.mlstorypatterns.StoryPatternLink;
import de.mdelab.mlstorypatterns.StoryPatternObject;

public class BreakpointDecoratorProvider extends AbstractProvider implements IDecoratorProvider
{

	@Override
	public boolean provides(IOperation operation)
	{
		if (operation instanceof CreateDecoratorsOperation)
		{
			IDecoratorTarget decoratorTarget = ((CreateDecoratorsOperation) operation).getDecoratorTarget();

			EditPart editPart = (EditPart) decoratorTarget.getAdapter(EditPart.class);

			if (editPart instanceof AbstractGraphicalEditPart
					&& (editPart instanceof IPrimaryEditPart || editPart instanceof CustomActivityEdgeGuardConstraintLabelEditPart
							|| editPart instanceof StoryPatternStoryPatternConstraintsCompartmentEditPart
							|| editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart || editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart))
			{
				View view = (View) editPart.getModel();

				EObject element = view.getElement();
				return element instanceof StoryNode || element instanceof StoryPatternObject || element instanceof ActivityEdge
						|| element instanceof StoryPatternLink || element instanceof DecisionMergeNode
						|| element instanceof InitialNode || element instanceof FlowFinalNode || element instanceof ActivityFinalNode
						|| element instanceof ExpressionActivityNode || element instanceof MLStringExpression;
			}
		}

		return false;

	}

	@Override
	public void createDecorators(IDecoratorTarget decoratorTarget)
	{
		EditPart editPart = (EditPart) decoratorTarget.getAdapter(EditPart.class);
		if (editPart != null
				&& (editPart instanceof IPrimaryEditPart || editPart instanceof CustomActivityEdgeGuardConstraintLabelEditPart
						|| editPart instanceof StoryPatternStoryPatternConstraintsCompartmentEditPart
						|| editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart || editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart))
		{
			BreakpointDecorator decorator = new BreakpointDecorator(decoratorTarget);

			decoratorTarget.installDecorator(MLSDMDebugUiConstants.BREAKPOINT_DECORATOR, decorator);
		}
	}

}
