package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.actions;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import de.mdelab.expressiondialogs.actions.CustomAbstractActionDelegate;
import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlcore.MLNamedElement;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.MLSDMDebugBreakpoint;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugUiConstants;
import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;

public class SetBreakpointAction extends CustomAbstractActionDelegate
{

	private EditPart	editPart;

	@Override
	protected void doRun(IProgressMonitor progressMonitor)
	{
		if (this.editPart != null)
		{
			EObject element = ((View) this.editPart.getModel()).getElement();
			String elementUUID = ((MLElementWithUUID) element).getUuid();
			String elementName;
			if (element instanceof MLNamedElement)
				elementName = ((MLNamedElement) element).getName() + " : " + element.eClass().getName();
			else
				elementName = (elementUUID + " : " + element.eClass().getName());
			
			IBreakpoint[] breakpoints = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER);

			for (IBreakpoint bp : breakpoints)
			{
				assert bp instanceof SDDebugBreakpoint;

				if (elementUUID.equals(((SDDebugBreakpoint) bp).getElementUUID()))
				{
					try
					{
						bp.delete();

						this.editPart.refresh();

						return;
					}
					catch (CoreException e)
					{
						throw new RuntimeException("Could not delete breakpoint of element '" + elementUUID + "'.", e);
					}
				}
			}

			try
			{
				String  uri = ((View)editPart.getModel()).eResource().getURI().toString();
				DebugPlugin.getDefault().getBreakpointManager().addBreakpoint(new MLSDMDebugBreakpoint(elementUUID, elementName, uri,  "", "OCL" ));
				this.editPart.refresh();
			}
			catch (Exception e)
			{
				throw new RuntimeException("Could not add breakpoint on element '" + elementUUID + "'.", e);
			}
		}
	}

	@Override
	public void selectionChanged(IAction act, ISelection selection)
	{
		if (selection instanceof IStructuredSelection)
		{
			this.editPart = (EditPart) ((IStructuredSelection) selection).getFirstElement();
		}
	}
}
