package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.PlatformUI;

import de.mdelab.expressiondialogs.MlDiagramConstants;
import de.mdelab.expressiondialogs.SourceViewerProvider;
import de.mdelab.mlexpressions.MlexpressionsPackage;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlstorypatterns.MlstorypatternsUtils;
import de.mdelab.mlstorypatterns.StoryPatternObject;

public class DebugbreakpointsView extends AbstractDebugView
{

	public DebugbreakpointsView()
	{
	}

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String		ID		= "de.mdelab.mlsdm.interpreter.debug.ui.BreakpointsView";

	MLSDMBreakpointsManager			manager	= MLSDMBreakpointsManager.getModelProvider();

	private CheckboxTableViewer		viewer;
	private TableColumn				elementStatusColumn;
	private TableColumn				elementNameColumn;
	private TableColumn				elementLocationColumn;
	private TableColumn				elementUUIDColumn;
	private Composite				composite;
	private Composite				upper, lower;
	private Composite				stringexpField;

	private SourceViewerProvider	currSourceVP;
	private LanguageSelectionWidget	expLanguageSelector;
	private EClassifier				contextClassifier;

	@Override
	public void createPartControl(final Composite parent)
	{

		this.VIEW_MAP.put(MlexpressionsPackage.Literals.ML_STRING_EXPRESSION.getName(), new StringExpressionView());

		parent.setLayout(new FillLayout(SWT.HORIZONTAL));

		this.composite = new Composite(parent, SWT.NONE);
		this.composite.setLayout(new FillLayout(SWT.HORIZONTAL));

		final SashForm sashForm = new SashForm(this.composite, SWT.NONE);
		sashForm.setOrientation(SWT.VERTICAL);

		final MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);

		this.upper = new Composite(sashForm, SWT.NONE);
		this.upper.setLayout(new FillLayout(SWT.HORIZONTAL));

		// Create the help context id for the viewer's control
		this.viewer = CheckboxTableViewer.newCheckList(this.upper, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION
				| SWT.BORDER | SWT.CHECK);

		PlatformUI.getWorkbench().getHelpSystem().setHelp(this.viewer.getControl(), "myTestView.viewer");

		final Table table = this.viewer.getTable();

		this.elementStatusColumn = new TableColumn(table, SWT.LEFT);
		this.elementStatusColumn.setResizable(false);
		this.elementStatusColumn.setMoveable(false);
		this.elementStatusColumn.setWidth(25);

		this.elementNameColumn = new TableColumn(table, SWT.LEFT);
		this.elementNameColumn.setText("Element name");
		this.elementNameColumn.setWidth(150);

		this.elementLocationColumn = new TableColumn(table, SWT.LEFT);
		this.elementLocationColumn.setText("Location");
		this.elementLocationColumn.setWidth(183);

		this.elementUUIDColumn = new TableColumn(table, SWT.LEFT);
		this.elementUUIDColumn.setText("UUID");
		this.elementUUIDColumn.setWidth(200);

		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		this.viewer.setContentProvider(new BreakpointViewContentProvider());
		this.viewer.setLabelProvider(new BreakpointViewLabelProvider());

		this.viewer.setCheckStateProvider(new ICheckStateProvider()
		{
			@Override
			public boolean isChecked(final Object element)
			{
				return ((ISDBreakpointItem) element).isEnabled();
			}

			@Override
			public boolean isGrayed(final Object element)
			{
				return false;
			}
		});

		this.viewer.setInput(this.manager);

		final Menu menu = menuMgr.createContextMenu(this.viewer.getControl());
		// Set the MenuManager
		this.viewer.getControl().setMenu(menu);
		this.getSite().registerContextMenu(menuMgr, this.viewer);
		// Make the selection available
		this.getSite().setSelectionProvider(this.viewer);

		this.viewer.addCheckStateListener(new ICheckStateListener()
		{
			@Override
			public void checkStateChanged(final CheckStateChangedEvent event)
			{
				final ISDBreakpointItem bp = (ISDBreakpointItem) event.getElement();
				bp.setEnabled(event.getChecked());
				bp.saveChanges();
			}
		});

		table.addListener(SWT.Selection, new Listener()
		{
			@Override
			public void handleEvent(final Event event)
			{
				final String string = event.detail == SWT.SELECTED ? "Checked" : "Selected";
				if (string.equals("Selected"))
				{
					if (DebugbreakpointsView.this.isDirty())
					{
						DebugbreakpointsView.this.doSave(null);
					}
					DebugbreakpointsView.this.currentItem = (ISDBreakpointItem) ((IStructuredSelection) (DebugbreakpointsView.this.viewer
							.getSelection())).getFirstElement();
					EObject eob = MLSDMBreakpointsManager.getModelProvider().getObject(DebugbreakpointsView.this.currentItem);

					if (eob instanceof StoryPatternObject)
					{
						DebugbreakpointsView.this.contextClassifier = ((StoryPatternObject) eob).getType();
					}
					else
					{
						DebugbreakpointsView.this.contextClassifier = null;
						// following commented code might be used in the future.
						// ComposedAdapterFactory af = new
						// ComposedAdapterFactory(ComposedAdapterFactory.Descriptor.Registry.INSTANCE);
						// NamedElementItemProvider p =
						// (NamedElementItemProvider) af.adapt(eob,
						// IItemPropertySource.class);
					}

					while (!(eob instanceof Activity))
					{
						if (eob.eContainer() != null)
						{
							eob = eob.eContainer();
						}
					}
					DebugbreakpointsView.this.activity = (Activity) eob;
					DebugbreakpointsView.this.changeLayout();
				}
			}
		});

		this.lower = new Composite(sashForm, SWT.NONE);
		this.lower.setLayout(new GridLayout(2, true));
		this.lower.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true, 1, 1));

		sashForm.setWeights(new int[]
		{
				1, 1
		});
		this.hookMouse();
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus()
	{
		this.viewer.getControl().setFocus();
	}

	private void hookMouse()
	{
		this.viewer.getTable().addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDoubleClick(final MouseEvent e)
			{
				final IStructuredSelection sel = (IStructuredSelection) DebugbreakpointsView.this.viewer.getSelection();
				final ISDBreakpointItem bp = (ISDBreakpointItem) sel.getFirstElement();
				DebugbreakpointsView.this.manager.openEditor(bp);
			}
		});
	}

	protected void changeLayout()
	{

		final Widget w = this.stringexpField;
		if ((w != null) && !w.isDisposed())
		{
			w.dispose();
		}

		if (this.sourceViewers != null)
		{
			for (final ISourceViewer sv : this.sourceViewers.values())
			{
				if (sv.getTextWidget() != null)
				{
					sv.getTextWidget().dispose();
				}
			}
		}

		if (this.currentItem.getExpressionLanguage() != null)
		{
			final String name = MlexpressionsPackage.Literals.ML_STRING_EXPRESSION.getName();

			if (this.VIEW_MAP.containsKey(name))
			{
				this.VIEW_MAP.get(name).adaptInputElements();
			}
		}

		this.lower.layout(true);
	}

	protected class LanguageSelectionRadioWidget extends Composite implements LanguageSelectionWidget
	{

		private final Collection<String>	languageDirectory;
		Composite							languageRadioArea;

		Map<Button, String>					buttonList;

		Vector<LanguageSelectedListener>	languageSelectedListeners	= new Vector<LanguageSelectedListener>();

		public LanguageSelectionRadioWidget(final Composite parent, final int style)
		{
			super(parent, style);
			this.setLayout(new RowLayout(SWT.VERTICAL));

			this.languageDirectory = new Vector<String>();

			final Label languageLabel = new Label(this, SWT.NONE);
			languageLabel.setText("Expression Language");
			this.createDialogLanguageRadioButtons(this);

		}

		@Override
		public void addLanguageSelectedListener(final LanguageSelectedListener listener)
		{
			this.languageSelectedListeners.addElement(listener);
		}

		@Override
		public void removeLanguageSelectedListener(final LanguageSelectedListener listener)
		{
			this.languageSelectedListeners.removeElement(listener);
		}

		public void createDialogLanguageRadioButtons(final Composite mainArea)
		{
			this.languageRadioArea = new Composite(mainArea, SWT.NONE);
			this.languageRadioArea.setLayout(new RowLayout(SWT.HORIZONTAL));
			this.buttonList = new HashMap<Button, String>();
		}

		protected void languageSelected(final String language)
		{
			final LanguageSelectedEvent e = new LanguageSelectedEvent(this, language);

			for (final LanguageSelectedListener listener : this.languageSelectedListeners)
			{
				listener.languageSelected(e);
			}
		}

		@Override
		public void addLanguage(final String language)
		{
			if (!this.languageDirectory.contains(language))
			{
				this.languageDirectory.add(language);
			}

			final Listener listener = new Listener()
			{
				@Override
				public void handleEvent(final Event e)
				{
					for (final Control aButton : LanguageSelectionRadioWidget.this.buttonList.keySet())
					{
						((Button) aButton).setSelection(false);
					}
					((Button) e.widget).setSelection(true);
					LanguageSelectionRadioWidget.this.languageSelected(LanguageSelectionRadioWidget.this.buttonList.get(e.widget));
				}
			};

			final Button tempButton = new Button(this.languageRadioArea, SWT.RADIO);
			tempButton.setText(language);
			tempButton.addListener(SWT.Selection, listener);
			final String buttonInfo = language;
			this.buttonList.put(tempButton, buttonInfo);
		}

		@Override
		public void setSelectedLanguage(final String language)
		{
			Button searchedButton = this.buttonList.keySet().iterator().next();
			for (final Button aButton : this.buttonList.keySet())
			{
				if (this.buttonList.get(aButton).equals(language))
				{
					searchedButton = aButton;
				}
				aButton.setSelection(false);
			}
			searchedButton.setSelection(true);
		}

		@Override
		public String getSelectedLanguage()
		{
			return this.getInfoOfSelectedButton();
		}

		private String getInfoOfSelectedButton()
		{
			String resultString = "none";
			for (final Button aButton : this.buttonList.keySet())
			{
				if (aButton.getSelection())
				{
					resultString = this.buttonList.get(aButton);
				}
			}
			return resultString;
		}
	}

	protected class LanguageSelectionComboWidget extends Composite implements LanguageSelectionWidget
	{

		Combo								languageCombo;

		Collection<String>					languageDirectory;

		Vector<LanguageSelectedListener>	languageSelectedListeners	= new Vector<LanguageSelectedListener>();

		public LanguageSelectionComboWidget(final Composite parent, final int style)
		{
			super(parent, style);
			this.setLayout(new GridLayout(2, false));

			this.languageDirectory = new Vector<String>();

			final Label languageLabel = new Label(this, SWT.NONE);
			languageLabel.setText("Expression Language");
			this.createDialogLanguageCombo(this);
		}

		@Override
		public void addLanguageSelectedListener(final LanguageSelectedListener listener)
		{

			this.languageSelectedListeners.addElement(listener);

		}

		@Override
		public void removeLanguageSelectedListener(final LanguageSelectedListener listener)
		{

			this.languageSelectedListeners.removeElement(listener);

		}

		private void createDialogLanguageCombo(final Composite mainArea)
		{
			this.languageCombo = new Combo(mainArea, SWT.DROP_DOWN);
			this.languageCombo.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, false, false));
			final String[] items = {};
			this.languageCombo.setItems(items);

			this.languageCombo.addSelectionListener(new SelectionListener()
			{

				@Override
				public void widgetSelected(final SelectionEvent e)
				{
					LanguageSelectionComboWidget.this.languageSelected(LanguageSelectionComboWidget.this.languageCombo
							.getItem(LanguageSelectionComboWidget.this.languageCombo.getSelectionIndex()));
				}

				@Override
				public void widgetDefaultSelected(final SelectionEvent e)
				{
					this.widgetSelected(e);
				}

			});
		}

		protected void languageSelected(final String language)
		{

			final LanguageSelectedEvent e = new LanguageSelectedEvent(this, language);

			for (final LanguageSelectedListener listener : this.languageSelectedListeners)
			{
				listener.languageSelected(e);
			}
		}

		@Override
		public void addLanguage(final String language)
		{
			final String[] items = {};
			this.languageDirectory.add(language);
			this.languageCombo.setItems(this.languageDirectory.toArray(items));
		}

		/**
		 * Sets the selected Language of the widget if it is already registered.
		 * If not nothing happens. Furthermore this does not trigger a
		 * languageSelectedEvent.
		 */
		@Override
		public void setSelectedLanguage(final String language)
		{
			final int index = this.searchStringInCombo(this.languageCombo, language);
			this.languageCombo.select(index);
		}

		private int searchStringInCombo(final Combo aCombo, final String searchString)
		{
			final String[] items = aCombo.getItems();
			final int numberOfItems = items.length;

			int searchedIndex = 0;
			for (int i = 0; (i < numberOfItems) && !items[searchedIndex].equals(searchString); i++)
			{
				searchedIndex++;
			}
			return searchedIndex;
		}

		@Override
		public String getSelectedLanguage()
		{
			return this.languageCombo.getItem(this.languageCombo.getSelectionIndex());
		}
	}

	@Override
	protected void addOrResetStringExpressionEditor(final String lang, final String expr)
	{
		final List<String> constraintLanguages = MlstorypatternsUtils.getAvailableExpressionLanguages();

		this.currSourceVP = this.sourceViewerProviders.get(lang);

		if ((constraintLanguages.size() == 0) || (this.currSourceVP == null))
		{
			return;
		}

		this.stringexpField = new Composite(this.lower, SWT.NONE);
		this.stringexpField.setLayout(new GridLayout(1, false));
		this.stringexpField.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));

		if (constraintLanguages.size() <= MlDiagramConstants.EXPRESSION_EDITOR_RADIO_BUTTON_THRESHOLD)
		{
			this.expLanguageSelector = new LanguageSelectionRadioWidget(this.stringexpField, SWT.NONE);
		}
		else
		{
			this.expLanguageSelector = new LanguageSelectionComboWidget(this.stringexpField, SWT.NONE);
		}
		for (final String aLanguage : constraintLanguages)
		{
			this.expLanguageSelector.addLanguage(aLanguage);
		}

		this.expLanguageSelector.addLanguageSelectedListener(new LanguageSelectedListener()
		{

			@Override
			public void languageSelected(final LanguageSelectedEvent event)
			{
				DebugbreakpointsView.this.currentItem.setExpressionLanguage(event.language);
				/*
				 * Make the current source viewer invisible
				 */
				DebugbreakpointsView.this.sourceViewers.get(event.language).getTextWidget().setVisible(false);
				((GridData) DebugbreakpointsView.this.sourceViewers.get(event.language).getTextWidget().getLayoutData()).exclude = true;

				DebugbreakpointsView.this.currSourceVP = DebugbreakpointsView.this.sourceViewerProviders.get(event.language);

				/*
				 * Set the text of the new source viewer
				 */
				DebugbreakpointsView.this.currSourceVP.setText(expr);

				/*
				 * Make the new source viewer visible
				 */
				DebugbreakpointsView.this.sourceViewers.get(event.language).getTextWidget().setVisible(true);
				((GridData) DebugbreakpointsView.this.sourceViewers.get(event.language).getTextWidget().getLayoutData()).exclude = false;

				DebugbreakpointsView.this.lower.layout();
			}
		});

		/*
		 * Create the source viewers for all source viewer providers
		 */

		this.sourceViewers = new HashMap<String, ISourceViewer>();

		for (final Entry<String, SourceViewerProvider> svpEntry : this.sourceViewerProviders.entrySet())
		{
			final ISourceViewer sv = svpEntry.getValue().createSourceViewer(this.lower, SWT.BORDER | SWT.MULTI, this.contextClassifier,
					this.getContextInformation(), "");
			this.sourceViewers.put(svpEntry.getKey(), sv);

			sv.getTextWidget().setVisible(false);

			final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
			gridData.exclude = true;

			sv.getTextWidget().setLayoutData(gridData);
		}

		// Select the expression language of the expression
		this.expLanguageSelector.setSelectedLanguage(lang);

		// Set the text of the source viewer
		this.currSourceVP.setText(expr);

		final StyledText st = this.sourceViewers.get(lang).getTextWidget();

		// Make the source viewer visible
		st.setVisible(true);
		((GridData) st.getLayoutData()).exclude = false;

		st.addListener(SWT.Modify, new Listener()
		{

			@Override
			public void handleEvent(final Event e)
			{
				DebugbreakpointsView.this.setDirty(true);
				DebugbreakpointsView.this.currentItem.setExpression(st.getText());
			}
		});

		this.lower.layout();
	}

}