package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarkerDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IBreakpointListener;
import org.eclipse.debug.core.IBreakpointManager;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditor;
import de.mdelab.mlsdm.interpreter.debug.ui.MLSDMDebugTarget;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.MLSDMDebugUiConstants;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugUiConstants;
import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;

public class MLSDMBreakpointsManager implements IBreakpointListener
{

	Logger												logger					= Logger.getLogger("ModelProvider");

	private static MLSDMBreakpointsManager				manager;

	private final Hashtable<String, ISDBreakpointItem>	breakpoints;

	private MLSDMDebugTarget							debugTarget				= null;

	private List<IBreakpointsManagerListener>			listenersToBeRemoved	= new ArrayList<IBreakpointsManagerListener>();

	// first string is a temporal ViewLocation, second String corresponding
	// model Location extracted from BP.
	private final Hashtable<String, String>				bpLocations;

	private final List<IBreakpointsManagerListener>		listeners				= new ArrayList<IBreakpointsManagerListener>();

	private MLSDMBreakpointsManager()
	{
		this.breakpoints = new Hashtable<String, ISDBreakpointItem>();
		this.bpLocations = new Hashtable<String, String>();
	}

	public static MLSDMBreakpointsManager getModelProvider()
	{

		if (MLSDMBreakpointsManager.manager == null)
		{
			MLSDMBreakpointsManager.manager = new MLSDMBreakpointsManager();
			MLSDMBreakpointsManager.manager.loadBreakPoints();
		}
		return MLSDMBreakpointsManager.manager;
	}

	public void setDebugTarget(final MLSDMDebugTarget debugTarget)
	{
		this.debugTarget = debugTarget;
	}

	public void resetDebugTarget()
	{
		this.debugTarget = null;
	}

	public void addListener(final IBreakpointsManagerListener listener)
	{
		if (!this.listeners.contains(listener))
		{
			this.listeners.add(listener);
		}
	}

	public void removeListener(final IBreakpointsManagerListener listener)
	{
		if (this.listeners.contains(listener))
		{
			this.listenersToBeRemoved.add(listener);
		}
	}

	public Object[] getBreakPointItems()
	{
		return this.breakpoints.values().toArray();
	}

	// initial load of SDE Breakpoints in the workspace
	private void loadBreakPoints()
	{
		final IBreakpointManager mgr = DebugPlugin.getDefault().getBreakpointManager();
		mgr.addBreakpointListener(this);

		final IBreakpoint[] bps = mgr.getBreakpoints(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER);

		for (final IBreakpoint bp : bps)
		{
			try
			{
				String location = (String) bp.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_URI);
				if (location.startsWith("platform:/resource/temporary"))
				{
					location = null;
				}
				final SDBreakPointItem bpItem = new SDBreakPointItem(bp);
				this.breakpoints.put((String) bp.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_UUID), bpItem);
			}
			catch (final CoreException e)
			{
				System.out.println("Breakpoint attributes are broken");
			}
		}
	}

	@Override
	public void breakpointAdded(final IBreakpoint breakpoint)
	{

		try
		{
			final String identifier = (String) breakpoint.getMarker().getAttribute(IBreakpoint.ID);

			if ((identifier != null) && identifier.equals(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER))
			{
				final ISDBreakpointItem bp = new SDBreakPointItem(breakpoint);
				this.breakpoints.put(bp.getUuid(), bp);
				this.changed(bp, BreakpointsManagerEvent.ADDED);
			}
		}
		catch (final CoreException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public void breakpointRemoved(final IBreakpoint breakpoint, final IMarkerDelta delta)
	{

		try
		{
			final String identifier = (String) breakpoint.getMarker().getAttribute(IBreakpoint.ID);
			if ((identifier != null) && identifier.equals(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER))
			{
				final String uuid = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_UUID);
				final ISDBreakpointItem bp = this.breakpoints.get(uuid);
				if (bp != null)
				{
					this.breakpoints.remove(uuid);
					this.changed(bp, BreakpointsManagerEvent.REMOVED);
				}
			}
		}
		catch (final CoreException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public void breakpointChanged(final IBreakpoint breakpoint, final IMarkerDelta delta)
	{

		try
		{
			final String identifier = (String) breakpoint.getMarker().getAttribute(IBreakpoint.ID);
			if ((identifier != null) && identifier.equals(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER))
			{
				final String uuid = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_UUID);
				final ISDBreakpointItem bp = this.breakpoints.get(uuid);
				if (bp != null)
				{
					this.changed(bp, BreakpointsManagerEvent.CHANGED);
				}
			}
		}
		catch (final CoreException e)
		{
			e.printStackTrace();
		}
	}

	// notify all listeners about BP change
	private void changed(final ISDBreakpointItem item, final int type)
	{

		// remove listeners which unregistered
		if (this.listenersToBeRemoved.size() > 0)
		{
			for (final IBreakpointsManagerListener l : this.listenersToBeRemoved)
			{
				this.listeners.remove(l);
			}
		}
		this.listenersToBeRemoved = new ArrayList<IBreakpointsManagerListener>();

		final BreakpointsManagerEvent event = new BreakpointsManagerEvent(this, item, type);
		for (final Iterator<IBreakpointsManagerListener> iter = this.listeners.iterator(); iter.hasNext();)
		{
			iter.next().breakpointChanged(event);
		}
	}

	// assign uri to a breakpoint if it was created during debug
	public void checkBreakpointLocation(final String modelLocation, final SDDebugBreakpoint breakpoint) throws CoreException
	{
		if (this.debugTarget != null)
		{
			if (!breakpoint.getElementURI().startsWith("platform:/resource/" + MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH)
					&& (this.bpLocations.get(modelLocation) == null))
			{
				this.bpLocations.put(modelLocation, breakpoint.getElementURI());
			}
			else if (this.bpLocations.get(modelLocation) != null)
			{
				final String uri = this.bpLocations.get(modelLocation);
				breakpoint.getMarker().setAttribute(SDDebugBreakpoint.ELEMENT_URI, uri);
				breakpoint.setElementURI(uri);
			}
		}
		else
		{
			if (breakpoint.getElementURI().equals(""))
			{
				breakpoint.getMarker().setAttribute(SDDebugBreakpoint.ELEMENT_URI, modelLocation);
				breakpoint.setElementURI(modelLocation);
			}
		}
	}

	public void removeBreakpoint(final String uuid)
	{
		final IBreakpointManager mgr = DebugPlugin.getDefault().getBreakpointManager();
		try
		{
			mgr.removeBreakpoint(this.breakpoints.get(uuid).getBreakpoint(), true);
		}
		catch (final CoreException e)
		{
			System.out.println("Breakpoint with uuid : " + uuid + "  does not exist");
		}
	}

	public void openEditor(final ISDBreakpointItem item)
	{
		if (this.debugTarget != null)
		{
			final EObject obj = this.debugTarget.getDebugClient().getSynchronizerAdapter().getEObjectUUIDManager()
					.getEObject(item.getUuid());
			this.debugTarget.getStoryDiagramEditorManager().focusDiagramElement((MLElementWithUUID) obj);
		}
		else
		{
			final String fileLocation = item.getElementURI();
			if (fileLocation != null)
			{
				this.focusDiagramElement(item);
			}
		}
	}

	public EObject getObject(final ISDBreakpointItem item)
	{

		EObject eObject = null;

		final String fileLocation = item.getElementURI();

		final IFile file = (IFile) ResourcesPlugin.getWorkspace().getRoot().findMember(fileLocation);

		final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

		IEditorPart editorPart;
		try
		{

			editorPart = IDE.openEditor(page, file, false);

			final MlsdmDiagramEditor storyDiagramEditor = (MlsdmDiagramEditor) editorPart;

			final Activity activity = (Activity) storyDiagramEditor.getDiagram().getElement();

			eObject = activity.eResource().getEObject(item.getUuid());

		}
		catch (final PartInitException e)
		{
			e.printStackTrace();
		}

		return eObject;
	}

	private void focusDiagramElement(final ISDBreakpointItem item)
	{

		/*
		 * Open the editor
		 */
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				try
				{
					final String fileLocation = item.getElementURI();

					final IFile file = (IFile) ResourcesPlugin.getWorkspace().getRoot().findMember(fileLocation);

					final IEditorPart editorPart = IDE.openEditor(page, file);

					final MlsdmDiagramEditor storyDiagramEditor = (MlsdmDiagramEditor) editorPart;

					final Activity activity = (Activity) storyDiagramEditor.getDiagram().getElement();

					final EObject eObject = activity.eResource().getEObject(item.getUuid());

					IGraphicalEditPart editPart = null;

					editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
							storyDiagramEditor.getDiagramEditPart(), eObject);

					assert editPart != null;

					storyDiagramEditor.getDiagramGraphicalViewer().reveal(editPart);

				}
				catch (final PartInitException e)
				{
					throw new RuntimeException("Could not open story diagram editor for '" + item.getElementURI() + "'.", e);
				}
			}
		});
	}
}