package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.model.IBreakpoint;

import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;

public class SDBreakPointItem implements ISDBreakpointItem {

	private String uuid;
	private boolean enabled;
	private String elementName;
	private String elementURI;
	private IBreakpoint breakpoint;
	private String expression;
	private String expressionLanguage;


	SDBreakPointItem(IBreakpoint breakpoint) {

		try {
			this.uuid =  (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_UUID);
			this.elementName = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_NAME);
			this.elementURI = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.ELEMENT_URI);
			this.expression = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.EXPRESSION);
			this.expressionLanguage = (String) breakpoint.getMarker().getAttribute(SDDebugBreakpoint.EXPRESSION_LANGUAGE);
			if (elementURI != null)
				this.elementURI = elementURI.substring("platform:/resource".length());
			this.breakpoint = breakpoint;
			if((Boolean) breakpoint.getMarker().getAttribute(IBreakpoint.ENABLED))
				this.enabled = true;
			else enabled = false;
		} catch (CoreException e) {
			System.out.println("Breakpoint is not a SDBreakpoint or attributes are missing");
		}
	}


	@Override
	public String getUuid() {
		return uuid;
	}
	
	@Override
	public String getElementName() {
		return elementName;
	}

	@Override
	public String getElementURI() {
		return elementURI;
	}

	@Override
	public String getExpressionLanguage() {
		return expressionLanguage;
	}

	@Override
	public String getExpression() {
		return expression;
	}
	
	@Override
	public boolean isEnabled() {
		return enabled;
	}

	@Override
	public IBreakpoint getBreakpoint() {
		return breakpoint;
	}

	@Override
	public void setExpressionLanguage(String expressionLanguage) {
		this.expressionLanguage = expressionLanguage;
	}
	
	@Override
	public void setEnabled(boolean checked) {
		this.enabled = checked;
	}

	@Override
	public void setExpression(String expr) {
		this.expression = expr;
	}

	@Override
	public String toString() {
		return elementName + " " + uuid;
	}


	@Override
	public void saveChanges() {
		try {
			breakpoint.getMarker().setAttribute(SDDebugBreakpoint.EXPRESSION, expression);
			breakpoint.getMarker().setAttribute(IBreakpoint.ENABLED, enabled);
			breakpoint.getMarker().setAttribute(SDDebugBreakpoint.EXPRESSION_LANGUAGE, expressionLanguage);
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
	}
}
