package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Polyline;
import org.eclipse.swt.graphics.Color;

public class ChangeFigureTransaction
{
	private static final int	LINE_WIDTH_NOT_SET	= -1;

	private final IFigure		figure;

	private final Border		oldBorder;

	private final Color			oldForegroundColor;

	private final int			oldLineWidth;

	public ChangeFigureTransaction(IFigure figure, Border newBorder)
	{
		assert newBorder != null;
		assert figure != null;

		this.figure = figure;

		this.oldBorder = figure.getBorder();
		figure.setBorder(newBorder);

		this.oldForegroundColor = null;
		this.oldLineWidth = LINE_WIDTH_NOT_SET;
	}

	public ChangeFigureTransaction(Polyline figure, Color newForegroundColor, int newLineWidth)
	{
		assert figure != null;
		assert newForegroundColor != null;
		assert newLineWidth >= 0;
		assert figure instanceof Polyline;

		this.oldBorder = figure.getBorder();
		this.oldForegroundColor = figure.getForegroundColor();
		this.oldLineWidth = figure.getLineWidth();

		this.figure = figure;

		figure.setForegroundColor(newForegroundColor);
		figure.setLineWidth(newLineWidth);
	}

	public void rollBack()
	{
		if (this.figure instanceof Polyline)
		{
			this.figure.setForegroundColor(this.oldForegroundColor);
			((Polyline) this.figure).setLineWidth(this.oldLineWidth);
		}
		else
		{
			this.figure.setBorder(this.oldBorder);
		}
	}
}
