package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;


import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.diagram.core.services.ViewService;
import org.eclipse.gmf.runtime.diagram.ui.resources.editor.document.IDiagramDocument;
import org.eclipse.gmf.runtime.diagram.ui.resources.editor.document.IDocument;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.ui.IEditorInput;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.diagram.edit.parts.ActivityEditPart;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditorPlugin;
import de.mdelab.mlsdm.diagram.part.MlsdmDocumentProvider;
//import de.mdelab.mlstorypatterns.diagram.part.MlstorypatternsDiagramEditorPlugin;

public class MLSDMDebuggerDocumentProvider extends MlsdmDocumentProvider
{
	@Override
	protected ElementInfo createElementInfo(Object element) throws CoreException
	{
		return super.createElementInfo(element);
	}

	@Override
	protected void setDocumentContent(IDocument document, IEditorInput element) throws CoreException
	{
		assert element instanceof MLSDMDebuggerEditorInput;

		MLSDMDebuggerEditorInput debuggerEditorInput = (MLSDMDebuggerEditorInput) element;

		IDiagramDocument diagramDocument = (IDiagramDocument) document;
		diagramDocument.setEditingDomain((TransactionalEditingDomain) debuggerEditorInput.getEditingDomain());

		final TransactionalEditingDomain domain = diagramDocument.getEditingDomain();

		/*
		 * Create a new story diagram if it does not exist
		 */
		if (!ExtensibleURIConverterImpl.INSTANCE.exists(debuggerEditorInput.getURI(), null))
		{
			ResourceSet rs = new ResourceSetImpl();
			final Resource diagramResource = rs.createResource(debuggerEditorInput.getURI());

			Activity activity = this.findActivity(debuggerEditorInput.getActivityUUID(), domain.getResourceSet());

			assert activity != null;
			
			final Diagram diagram = ViewService.createDiagram(activity, ActivityEditPart.MODEL_ID,
					MlsdmDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT);

			diagramResource.getContents().add(diagram);

			try
			{
				diagramResource.save(null);
			}
			catch (IOException e)
			{
				throw new RuntimeException("Could not save temporary story diagram '" + debuggerEditorInput.getURI().toString() + "'.", e);
			}
		}

		super.setDocumentContent(diagramDocument, element);
	}

	private Activity findActivity(String activityUUID, ResourceSet resourceSet)
	{
		TreeIterator<Notifier> it = resourceSet.getAllContents();

		while (it.hasNext())
		{
			Notifier notifier = it.next();

			if (notifier instanceof Activity && ((Activity) notifier).getUuid().equals(activityUUID))
			{
				return (Activity) notifier;
			}
		}

		return null;
	}

	@Override
	protected void disposeElementInfo(Object element, ElementInfo info)
	{
		assert element instanceof MLSDMDebuggerEditorInput;

		MLSDMDebuggerEditorInput editorInput = (MLSDMDebuggerEditorInput) element;

		if (editorInput.getSynchronizerAdapter() != null)
		{
			try
			{
				editorInput.getSynchronizerAdapter().shutdown();
			}
			catch (Exception e)
			{
				throw new RuntimeException("Could not shutdown synchronizer adapter.", e);
			}
		}

		super.disposeElementInfo(element, info);
	}
}
