package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;

import java.io.IOException;
import java.net.ServerSocket;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Polyline;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.diagram.edit.parts.ActivityEdgeEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.MapEntryLinkEditPart;
import de.mdelab.mlsdm.diagram.edit.parts.StoryPatternLinkEditPart;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditor;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.MLSDMDebugUiConstants;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugTarget;
import de.mdelab.sdm.interpreter.code.debug.ui.storyDiagramEditor.SDEditorManager;
import de.mdelab.sdm.interpreter.core.notifications.NotificationTypeEnum;

public class MLSDMEditorManager extends SDEditorManager<MLElementWithUUID>
{

	private ChangeFigureTransaction	lastChangeFigureTransaction;

	public MLSDMEditorManager(final SDDebugTarget<MLElementWithUUID, Activity> debugTarget)
	{
		super(debugTarget);
	}

	@Override
	public void focusDiagramElement(final MLElementWithUUID element)
	{

		/*
		 * Attach synchronizer adapter to the element's resource
		 */
		final MLSDMDebuggerEditorInput editorInput = this.getEditorInput(element);

		/*
		 * Open the editor
		 */
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				try
				{
					final IEditorPart editorPart = page.openEditor(editorInput, MLSDMDebuggerStoryDiagramEditor.ID);

					final MLSDMDebuggerStoryDiagramEditor storyDiagramEditor = (MLSDMDebuggerStoryDiagramEditor) editorPart;

					/*
					 * Get the corresponding element in the diagram editor's
					 * resource. Note, that the provided element is located in
					 * the instance graph editor's resource. We have to find the
					 * corresponding element in the diagram editor's resource
					 * using the element's UUID.
					 */

					final Activity activity = (Activity) ((MlsdmDiagramEditor) storyDiagramEditor).getDiagram().getElement();

					final EObject eObject = activity.eResource().getEObject(element.getUuid());

					IGraphicalEditPart editPart = null;

					editPart = (IGraphicalEditPart) ((MlsdmDiagramEditor) storyDiagramEditor).getDiagramEditPart().findEditPart(
							((MlsdmDiagramEditor) storyDiagramEditor).getDiagramEditPart(), eObject);

					assert editPart != null;

					((MlsdmDiagramEditor) storyDiagramEditor).getDiagramGraphicalViewer().reveal(editPart);

				}
				catch (final PartInitException e)
				{
					throw new RuntimeException("Could not open story diagram editor for '" + editorInput.getURI() + "'.", e);
				}
			}
		});
	}

	@Override
	public void highlightStoryDiagramElement(final MLElementWithUUID element, final NotificationTypeEnum notification)
	{
		/*
		 * Attach synchronizer adapter to the element's resource
		 */
		final MLSDMDebuggerEditorInput editorInput = this.getEditorInput(element);

		this.focusDiagramElement(element);
		/*
		 * Open the editor
		 */
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				try
				{

					final IEditorPart editorPart = page.openEditor(editorInput, MLSDMDebuggerStoryDiagramEditor.ID);

					final MLSDMDebuggerStoryDiagramEditor storyDiagramEditor = (MLSDMDebuggerStoryDiagramEditor) editorPart;

					/*
					 * Get the corresponding element in the diagram editor's
					 * resource. Note, that the provided element is located in
					 * the instance graph editor's resource. We have to find the
					 * corresponding element in the diagram editor's resource
					 * using the element's UUID.
					 */

					final Activity activity = (Activity) storyDiagramEditor.getDiagram().getElement();

					EObject eObject = activity.eResource().getEObject(element.getUuid());

					IGraphicalEditPart editPart = null;

					if ((eObject instanceof ActivityEdge) || (eObject instanceof AbstractStoryPatternLink))
					{
						for (final Object object : storyDiagramEditor.getDiagramEditPart().getConnections())
						{
							assert object instanceof ConnectionEditPart;

							if (((View) ((ConnectionEditPart) object).getModel()).getElement() == eObject)
							{
								editPart = (IGraphicalEditPart) object;
								break;
							}
						}
					}
					else if (eObject instanceof MLExpression)
					{
						editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
								storyDiagramEditor.getDiagramEditPart(), eObject);

						while (editPart == null)
						{
							eObject = eObject.eContainer();
							editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
									storyDiagramEditor.getDiagramEditPart(), eObject);
						}
					}
					else
					{
						editPart = (IGraphicalEditPart) storyDiagramEditor.getDiagramEditPart().findEditPart(
								storyDiagramEditor.getDiagramEditPart(), eObject);
					}

					assert editPart != null;

					// storyDiagramEditor.getDiagramGraphicalViewer().reveal(editPart);

					/*
					 * Undo the last change figure transaction
					 */
					MLSDMEditorManager.this.rollBackLastChangeFigureTransaction();

					/*
					 * if it is a link, make it thicker and colorize it
					 */
					if ((editPart instanceof ActivityEdgeEditPart) || (editPart instanceof StoryPatternLinkEditPart)

					|| (editPart instanceof de.mdelab.mlstorypatterns.diagram.edit.parts.StoryPatternLinkEditPart)
							|| (editPart instanceof de.mdelab.mlstorypatterns.diagram.edit.parts.StoryPatternLinkEditPart)
							|| (editPart instanceof MapEntryLinkEditPart))
					{
						final Polyline polyline = (Polyline) ((AbstractGraphicalEditPart) editPart).getFigure();

						MLSDMEditorManager.this.setLastChangeFigureTransaction(new ChangeFigureTransaction(polyline,
								MLSDMEditorManager.this.getColorForNotification(notification), 3));
					}
					/*
					 * Change the border of nodes
					 */
					else
					{
						MLSDMEditorManager.this.setLastChangeFigureTransaction(new ChangeFigureTransaction(
								((AbstractGraphicalEditPart) editPart).getFigure(), new LineBorder(MLSDMEditorManager.this
										.getColorForNotification(notification), 3)));
					}
				}
				catch (final PartInitException e)
				{
					throw new RuntimeException("Could not open story diagram editor for '" + editorInput.getURI() + "'.", e);
				}
			}
		});
	}

	private void setLastChangeFigureTransaction(final ChangeFigureTransaction lastChangeFigureTransaction)
	{
		this.lastChangeFigureTransaction = lastChangeFigureTransaction;
	}

	private void rollBackLastChangeFigureTransaction()
	{
		if (this.lastChangeFigureTransaction != null)
		{
			this.lastChangeFigureTransaction.rollBack();

			this.lastChangeFigureTransaction = null;
		}
	}

	@Override
	public MLSDMDebuggerEditorInput getEditorInput(final MLElementWithUUID element)
	{
		/*
		 * Check if an editor is already open that contains the element
		 */
		EObject activity = element;

		while (!(activity instanceof Activity) && (activity != null))
		{
			activity = activity.eContainer();
		}

		assert activity != null;
		assert activity instanceof Activity;

		final URI diagramURI = this.getDiagramURI((Activity) activity);

		MLSDMDebuggerEditorInput editorInput = null;
		if (activity instanceof MLElementWithUUID)
		{
			editorInput = new MLSDMDebuggerEditorInput(diagramURI, this.getDebugTarget().getDebuggerEditingDomain(),
					((MLElementWithUUID) activity).getUuid());
		}

		return editorInput;
	}

	protected URI getDiagramURI(final Activity activity)
	{
		this.openTempProject();

		final URI diagramURI = URI.createPlatformResourceURI(
				"/" + MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH + "/" + activity.getUuid() + ".mlsdm_diagram", false);

		/*
		 * Delete the story diagram if it exists but is too old.
		 */
		if (URIConverter.INSTANCE.exists(diagramURI, null))
		{
			final Map<String, ?> attributes = URIConverter.INSTANCE.getAttributes(diagramURI, null);

			final long timestamp = (Long) attributes.get(URIConverter.ATTRIBUTE_TIME_STAMP);

			if (timestamp < activity.eResource().getTimeStamp())
			{
				try
				{
					URIConverter.INSTANCE.delete(diagramURI, null);
				}
				catch (final IOException e)
				{
					throw new RuntimeException("Could not delete file '" + diagramURI.toString() + "'.", e);
				}
			}
		}

		return diagramURI;
	}

	private void openTempProject()
	{
		final IWorkspace workspace = ResourcesPlugin.getWorkspace();
		final IWorkspaceRoot root = workspace.getRoot();
		final IProject project = root.getProject(MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH);

		try
		{
			if (!project.exists())
			{
				project.create(new NullProgressMonitor());
			}

			project.open(new NullProgressMonitor());
		}
		catch (final CoreException e)
		{
			throw new RuntimeException("Could not open project '" + MLSDMDebugUiConstants.TEMPORARY_STORY_DIAGRAMS_PATH + "'.", e);
		}
	}

	public static int findOpenPort(final int startPort)
	{
		int port = startPort;

		while (port < Integer.MAX_VALUE)
		{
			try
			{
				final ServerSocket serverSocket = new ServerSocket(port);
				serverSocket.close();

				System.out.println("Found open port " + port);

				return port;
			}
			catch (final IOException e)
			{
				port++;
			}
		}

		return -1;
	}

	@Override
	public void closeAllStoryDiagramEditors()
	{
		Display.getDefault().syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				MLSDMEditorManager.this.rollBackLastChangeFigureTransaction();

				final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

				final IEditorReference[] editors = page.findEditors(null, MLSDMDebuggerStoryDiagramEditor.ID, IWorkbenchPage.MATCH_ID);

				page.closeEditors(editors, true);
			}
		});
	}
}
